searchData={"items":[{"type":"behaviour","title":"PlugCaisson","doc":"Body reader for supporting compressed `Plug` requests.","ref":"PlugCaisson.html"},{"type":"behaviour","title":"Implementing algorithm - PlugCaisson","doc":"In addition to the built in algorithms (see `read_body/2`) it is possible to\nimplement custom algorithms by implementing behaviour defined in this module.\n\n#","ref":"PlugCaisson.html#module-implementing-algorithm"},{"type":"behaviour","title":"Example - PlugCaisson","doc":"```elixir\ndefmodule BobbyCompression do\n  @behaviour PlugCaisson\n\n  # Initialise decompression state with whatever is needed\n  @impl true\n  def init(opts), do: {:ok, Bobby.open()}\n\n  # Gracefully close the state\n  @impl true\n  def deinit(state), do: Bobby.finish(state)\n\n  # Process read data to decompress them\n  @impl true\n  def process(state, data, _opts) do\n    case Bobby.decompress(state, data) do\n      {:finished, decompressed, new_state} ->\n        # All data was decompressed and there is no more data to be\n        # decompressed in stream\n        {:ok, decompressed, new_state}\n\n      {:more, decompressed, new_state} ->\n        # It can happen that `decompressed == \"\"` and this is perfectly fine\n        # as long as there is possibility that there will be more data in\n        # future\n        {:more, decompressed, new_state}\n\n      {:error, _} = error -> error\n    end\n  end\nend\n```","ref":"PlugCaisson.html#module-example"},{"type":"callback","title":"PlugCaisson.deinit/1","doc":"Cleanup for the state. This will be called in\n`Plug.Conn.register_before_send/2` callback, so the same conditions as with\nthese apply. It is guaranteed that it will be called only once for each\nconnection.","ref":"PlugCaisson.html#c:deinit/1"},{"type":"callback","title":"PlugCaisson.init/1","doc":"Initialise state for the decompression algorithm.\n\nThis callback will be called if and only if given algorithm was picked as a\nsuitable option for decompression. The returned state will be stored in the\n`Plug.Conn.t()`. It is guaranteed that it will be called only on first call to\n`read_body/2` and all subsequent calls will not call this function again.\n\nIt will receive data passed as a second value in tuple declared in the\nalgorithm map.","ref":"PlugCaisson.html#c:init/1"},{"type":"callback","title":"PlugCaisson.process/3","doc":"Process chunk of data.\n\nIt receives current state, binary read from the request and list of options\npassed to the `read_body/2` as a 2nd argument.","ref":"PlugCaisson.html#c:process/3"},{"type":"callback","title":"Return value - PlugCaisson.process/3","doc":"In case of success it should return 3-ary tuple:\n\n- `{:ok, binary(), new_state :: term()}` - wich mean that all data was read\n  and there is no more data left in the internal buffer.\n- `{:more, binary(), new_state :: term()}` - which mean that data was\n  processed, but there is more data left to be read in future calls.\n\nIf error occured during processing `{:error, term()}` tuple should be returned.","ref":"PlugCaisson.html#c:process/3-return-value"},{"type":"function","title":"PlugCaisson.read_body/2","doc":"Read `Plug.Conn` request body and decompress it if needed.","ref":"PlugCaisson.html#read_body/2"},{"type":"function","title":"Options - PlugCaisson.read_body/2","doc":"Accepts the same set of options as `Plug.Conn.read_body/2` with one option\nextra: `:algorithms` which is map containing algorithm identifier as key and\ntuple containing module name for module that implements `PlugCaisson`\nbehaviour and value that will be passed as 2nd argument to the `c:init/1`\ncallback.\n\nBy default the value is set to:\n\n```\n%{\n  \"br\" => {PlugCaisson.Brotli, []},\n  \"deflate\" => {PlugCaisson.Zlib, [type: :deflate]},\n  \"gzip\" => {PlugCaisson.Zlib, [type: :gzip]},\n  \"zstd\" => {PlugCaisson.Zstandard, []}\n}\n```","ref":"PlugCaisson.html#read_body/2-options"},{"type":"function","title":"Supported algorithms - PlugCaisson.read_body/2","doc":"- `gzip`\n- `deflate`\n- `br` (Brotli) - only if `:brotli` dependency is available\n- `zstd` (Zstandard) - only if `:ezstd` dependency is available","ref":"PlugCaisson.html#read_body/2-supported-algorithms"},{"type":"function","title":"Options - PlugCaisson.read_body/2","doc":"All passed opts will be passed to `Plug.Conn.read_body/2` and to used\ndecompression handlers. Decompressors by default will use `:length` to limit\namount of returned data to prevent zipbombs. Returned data can be longer than\n`:length` if the internal decompression buffer was larger. As it is described\nin `Plug.Conn.read_body/2` docs. By default `length: 8_000_000`.","ref":"PlugCaisson.html#read_body/2-options"},{"type":"module","title":"PlugCaisson.Brotli","doc":"Implementation for [Brotli][] compression.\n\n[Brotli]: https://brotli.org","ref":"PlugCaisson.Brotli.html"},{"type":"module","title":"PlugCaisson.Zlib","doc":"Implementation for `DEFLATE`/Zlib based compression methods:\n\n- `gzip`\n- `deflate`\n\nThese two are the most common algorithms used in the Web.","ref":"PlugCaisson.Zlib.html"},{"type":"module","title":"Options - PlugCaisson.Zlib","doc":"- `:window_bits` - size of the decompression window\n- `:type` - either `:gzip` or `:deflate` that will set proper `:window_bits`\n  according to each algorithm\n\nIt is preferred to use `:type` over `:window_bits`, but if both are specified,\nthen `:window_bits` take precedence.","ref":"PlugCaisson.Zlib.html#module-options"},{"type":"module","title":"PlugCaisson.Zstandard","doc":"Implementation for [Zstandard][zstd] compression.\n\n[zstd]: https://facebook.github.io/zstd/","ref":"PlugCaisson.Zstandard.html"},{"type":"extras","title":"PlugCaisson","doc":"# PlugCaisson\n\nAn Elixir `Plug.Parsers` body reader that supports compressed payloads. Supports gzip, brotli, and [more](#supported-algorithms).\n\n> A [caisson][caisson] is an geoengineering structure used to work on foundations of\n> bridges and piers. Due to pressurised environment in the caisson (required to\n> keep water out of it (as it has no bottom) people can get [decompression\n> sickness][sick], also known as _caisson disease_.\n\n[sick]: https://en.wikipedia.org/wiki/Decompression_sickness\n[caisson]: https://en.wikipedia.org/wiki/Caisson_(engineering)","ref":"readme.html"},{"type":"extras","title":"Installation - PlugCaisson","doc":"Add `:plug_caisson` to dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:plug_caisson, \"~> 0.2.0\"},\n    # optional, for brotli support\n    {:brotli, \"~> 0.3.2\"},\n    # optional, for zstd support\n    {:ezstd, \"~> 1.0\"},\n  ]\nend\n```\n\nAnd then add `{PlugCaisson, :read_body, []}` as a `:body_reader` to your `Plug.Parsers`:\n\n```elixir\nplug Plug.Parsers,\n  parsers: [:urlencoded, :json],\n  body_reader: {PlugCaisson, :read_body, []},\n  length: 8_000_000\n```\n\nAll options passed to the plug are forwarded to [`Plug.Conn.read_body/2`](https://hexdocs.pm/plug/Plug.Conn.html#read_body/2).\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Supported algorithms - PlugCaisson","doc":"- `gzip` - GNU zip - most common compression algorithm used in HTTP - [RFC 1952][]\n- `deflate` - DEFLATE compressed data ([RFC 1951][]) wrapped in Zlib data format\n  ([RFC 1950][])\n- `br` (requires optional `:brotli` dependency) - Brotli algorithm developed by\n  Google and supported by most browsers nowadays - [RFC 7932][]\n- `zstd` (requires optional `:ezstd` dependency)[^1] - Zstandard algorithm developed\n  by Meta (Facebook) which provides faster compression/decompression times than\n  Brotli, but worse compression ratio - [RFC 8478][]\n\n[RFC 1950]: https://datatracker.ietf.org/doc/html/rfc1950\n[RFC 1951]: https://datatracker.ietf.org/doc/html/rfc1951\n[RFC 1952]: https://datatracker.ietf.org/doc/html/rfc1952\n[RFC 7932]: https://datatracker.ietf.org/doc/html/rfc7932\n[RFC 8478]: https://datatracker.ietf.org/doc/html/rfc8478\n\n[^1]:\n    Do not support streaming decoding yet, because of lack support for such\n    flow upstream, see [ezstd#11](https://github.com/silviucpp/ezstd/issues/11)","ref":"readme.html#supported-algorithms"}],"content_type":"text/markdown"}